/*
 * Fichero: ArrastrarGrafico.java
 * Autor: Chuidiang
 * Fecha: 6/06/07 21:40
 */
package chuidiang.ejemplos.arrastrar_grafico;

import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;

import java.util.LinkedList;


/**
 * Ejemplo de grfico que se puede arrastrar con el ratn.
 *
 * @author Chuidiang
 *
 */
public class LienzoParaArrastrarFiguras extends Canvas implements MouseMotionListener
{
    /**
     * serial uid
     */
    private static final long serialVersionUID = -4273648398171436938L;

    /** 
     * Lista de figuras a dibujar.
     */
    private LinkedList<InterfaceFigura> listaFiguras = new LinkedList<InterfaceFigura>();

    /**
     * Si actualmente se est arrastrando o no el rectngulo.
     */
    private InterfaceFigura figuraArrastrandose = null;

    /**
     * x en la que estaba anteriormente el ratn.
     */
    private int xAnteriorRaton;

    /**
     * y en la que estaba anteriormente el ratn
     */
    private int yAnteriorRaton;

    /**
     * Crea un nuevo objeto ArrastrarGrafico.
     */
    public LienzoParaArrastrarFiguras()
    {
        addMouseMotionListener(this);
    }

    /**
     * Aada una figura a la lista de figuras a dibujar
     *
     * @param figura Una nueva figura a dibujar
     */
    public void addFigura(InterfaceFigura figura)
    {
        listaFiguras.add(figura);
    }

    /**
     * Quita la figura en la lista de figuras a dibujar.
     *
     * @param figura figura a quitar de la lista.
     */
    public void removeFigura(InterfaceFigura figura)
    {
        listaFiguras.remove(figura);
    }

    /**
     * Para darle un tamao por defecto al Canvas de dibujo
     *
     * @return Dimension por defecto.
     */
    @Override
    public Dimension getPreferredSize()
    {
        return new Dimension(500, 500);
    }

    /**
     * Dibuja el rectngulo en la posicin indicada por por xRectangulo e
     * yRectangulo.
     *
     * @param g Graphics con el que dibujar.
     */
    @Override
    public void paint(Graphics g)
    {
        for (InterfaceFigura figura : listaFiguras)
        {
            figura.dibujate(g);
        }
    }

    /**
     * Mtodo al que se llama cuando se arrastra el ratn.
     * Se comprueba con el atributo arrastrando si est empezando el arrastre o
     * ya se esta en medio del mismo.
     * Si se comienza el arrastre, se guardan las coordenadas del ratn que
     * vienen en el evento MouseEvent y se cambia el valor del atributo arrastrando.
     * Si se est en medio de un arrastre, se calcula la nueva posicin del
     * rectngulo y se llama al mtodo repaint() para que se pinte.
     *
     * @param e Evento del ratn
     */
    public void mouseDragged(MouseEvent e)
    {
        // Si comienza el arrastre ...
        if (figuraArrastrandose == null)
        {
            // Se guardan las posiciones del ratn
            xAnteriorRaton = e.getX();
            yAnteriorRaton = e.getY();
            // y se marca que ha comenzado el arrastre.
            figuraArrastrandose = dameFigura(e);
        }
        else
        {
            // Si ya haba empezado el arrastre, se calculan las nuevas
            // coordenadas del rectngulo
            figuraArrastrandose.setPosicion(
                figuraArrastrandose.getX() + (e.getX() - xAnteriorRaton),
                figuraArrastrandose.getY() + (e.getY() - yAnteriorRaton));

            // Se guarda la posicin del ratn para el siguiente clculo
            xAnteriorRaton = e.getX();
            yAnteriorRaton = e.getY();

            // y se manda repintar el Canvas
            repaint();
        }
    }

    /**
     * Para ver si el ratn est dentro del rectngulo.
     * Si est dentro, puede comenzar el arrastre.
     *
     * @param e El evento de ratn
     *
     * @return true si el ratn est dentro del rectngulo
     */
    private InterfaceFigura dameFigura(MouseEvent e)
    {
        for (InterfaceFigura figura : listaFiguras)
        {
            if (figura.estaDentro(e.getX(), e.getY()))
            {
                return figura;
            }
        }

        return null;
    }

    /**
     * El ratn se mueve sin arrastrar. Se marca fin de arrastre.
     *
     * @param e E
     */
    public void mouseMoved(MouseEvent e)
    {
        figuraArrastrandose = null;
    }
}
